// shape.cc	v1.3	9. June 2007	by eddy

/*  lego -- this program generates images of LEGO(R)-like models.

    		Copyright (C) 1998 - 2008 Edmund Nussbaum.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that you will have a lot of 
    fun when using it, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
    the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Please report your experiences, suggestions or whatever related
    to this program to: 

    sibylle.eddy @ t-online.de
    en @ mpifr-bonn.mpg.de 

    or send mail to:
    Edmund Nussbaum, Richard-Wagner-Str. 59, 53115 Bonn, Germany 
*/

#include "shape.h"
#include "param.h"
#include <stdlib.h>

shape::shape(param& p)
	{
	sha_org = p.org();
	sha_siz = p.siz();
	sha_col = p.col();
	sha_rot = p.rot();
	}

/////////////////////////////////////////////////////////////////////////////

static vector circularXZ(double phi)
	{
	phi *= DEGREE;
	return vector( sin(phi), 0, 2.5*cos(phi) );
	}

static vector circularYZ(double phi)
	{
	phi *= DEGREE;
	return vector( 0, sin(phi), 2.5*cos(phi) );
	}

////////////////////////////////////////////////////////////////////////////
// SHAPES:
//////////

stud1::stud1(const vector& rel,param& p)
	{
	sha_org = rel+p.org();
	sha_siz = vector(0,0,0.6);
	sha_col = p.col();
	sha_rot = p.rot();
	}

void stud1::draw(room& r)
	{
	vector top(sha_org+sha_siz);
	r.circle(sha_org,sha_siz,0.3,OUTLINE,sha_rot);
	r.circle(top,sha_siz,0.3,OUTLINE,sha_rot);
	r.disc(top,sha_siz,0.3,sha_col,sha_rot);
	r.cyl(sha_org,sha_siz,0.3,sha_col.dark(),sha_rot);
	}

stud2::stud2(const vector& rel,param& p)
	{
	sha_org = rel+p.org();
	sha_siz = vector(0,0,0.6);
	sha_col = p.col();
	sha_rot = p.rot();
	}

void stud2::draw(room& r)
	{
	color DARK(sha_col.dark());
	vector top(sha_org+sha_siz);
	r.ring(top,sha_siz,0.18,0.3,sha_col,sha_rot);
	r.circle(sha_org,sha_siz,0.3,OUTLINE,sha_rot);
	r.circle(sha_org,sha_siz,0.18,OUTLINE,sha_rot);
	r.circle(top,sha_siz,0.3,OUTLINE,sha_rot);
	r.circle(top,sha_siz,0.18,OUTLINE,sha_rot);
	r.cyl(sha_org,sha_siz,0.3,DARK,sha_rot);
	r.cyl(sha_org,sha_siz,0.18,DARK,sha_rot);
	}

void body::draw(room& r)
	{
	double x = sha_siz.x();
	double y = sha_siz.y();
	double z = sha_siz.z();
	vector a = sha_org;
	vector b = sha_org + vector(x,0,0);
	vector c = sha_org + vector(0,y,0);
	vector d = sha_org + vector(x,y,0);
	vector e = sha_org + vector(0,0,z);
	vector f = sha_org + vector(x,0,z);
	vector g = sha_org + vector(0,y,z);
	vector h = sha_org + vector(x,y,z); 
	r.area(a,b,e,sha_col,sha_rot);
	r.area(a,c,e,sha_col,sha_rot);
	r.area(b,d,f,sha_col,sha_rot);
	r.area(c,d,g,sha_col,sha_rot);
	r.area(e,f,g,sha_col,sha_rot);
	r.line(a,b,OUTLINE,sha_rot);
	r.line(e,f,OUTLINE,sha_rot);
	r.line(a,e,OUTLINE,sha_rot);
	r.line(b,f,OUTLINE,sha_rot);
	r.line(a,c,OUTLINE,sha_rot);
	r.line(b,d,OUTLINE,sha_rot);
	r.line(e,g,OUTLINE,sha_rot);
	r.line(f,h,OUTLINE,sha_rot);
	r.line(c,d,OUTLINE,sha_rot);
	r.line(g,h,OUTLINE,sha_rot);
	r.line(c,g,OUTLINE,sha_rot);
	r.line(d,h,OUTLINE,sha_rot);
	}

void groundbody::draw(room& r)
	{
	const double R = 0.3;
	double x = sha_siz.x();
	double y = sha_siz.y();
	vector z(0,0,sha_siz.z());
	vector a = sha_org + vector(  R,  R,0);
	vector b = sha_org + vector(x-R,  R,0);
	vector c = sha_org + vector(x-R,y-R,0);
	vector d = sha_org + vector(  R,y-R,0);
	vector e = sha_org + vector( 0 ,  R,0);
	vector f = sha_org + vector(  R, 0 ,0);
	vector g = sha_org + vector(x-R, 0 ,0);
	vector h = sha_org + vector(x  ,  R,0);
	vector i = sha_org + vector(x  ,y-R,0);
	vector j = sha_org + vector(x-R,y  ,0);
	vector k = sha_org + vector(  R,y  ,0);
	vector l = sha_org + vector( 0 ,y-R,0);
 	r.area(e,h,l,sha_col,sha_rot);
	r.area(f,g,a,sha_col,sha_rot);
	r.area(d,c,k,sha_col,sha_rot);
	r.area(f,f-z,g,sha_col,sha_rot);
	r.area(h,h-z,i,sha_col,sha_rot);
	r.area(j,j-z,k,sha_col,sha_rot);
	r.area(l,l-z,e,sha_col,sha_rot);
	r.sector(a,z,e-a,90,sha_col,sha_rot);
	r.sector(b,z,g-b,90,sha_col,sha_rot);
	r.sector(c,z,i-c,90,sha_col,sha_rot);
	r.sector(d,z,k-d,90,sha_col,sha_rot);
	r.cylsector(a-z,z,e-a,90,sha_col,sha_rot);
	r.cylsector(b-z,z,g-b,90,sha_col,sha_rot);
	r.cylsector(c-z,z,i-c,90,sha_col,sha_rot);
	r.cylsector(d-z,z,k-d,90,sha_col,sha_rot);
	r.line(f,g,OUTLINE,sha_rot);
	r.line(h,i,OUTLINE,sha_rot);
	r.line(j,k,OUTLINE,sha_rot);
	r.line(l,e,OUTLINE,sha_rot);
	r.line(f-z,g-z,OUTLINE,sha_rot);
	r.line(h-z,i-z,OUTLINE,sha_rot);
	r.line(j-z,k-z,OUTLINE,sha_rot);
	r.line(l-z,e-z,OUTLINE,sha_rot);
	r.arc(a  ,z,e-a,90,OUTLINE,sha_rot);
	r.arc(a-z,z,e-a,90,OUTLINE,sha_rot);
	r.arc(b  ,z,g-b,90,OUTLINE,sha_rot);
	r.arc(b-z,z,g-b,90,OUTLINE,sha_rot);
	r.arc(c  ,z,i-c,90,OUTLINE,sha_rot);
	r.arc(c-z,z,i-c,90,OUTLINE,sha_rot);
	r.arc(d  ,z,k-d,90,OUTLINE,sha_rot);
	r.arc(d-z,z,k-d,90,OUTLINE,sha_rot);
	}

void antiroofbody::draw(room& r)
	{
	double x = sha_siz.x();
	double y = sha_siz.y();
	double z = sha_siz.z();
	vector lr(x,0,0);
	vector al = sha_org;
	vector bl = sha_org + vector(0,0,z);
	vector cl = sha_org + vector(0,y,z);
	vector dl = sha_org + vector(0,y,z-0.63);
	vector el = sha_org + vector(0,1,0);
	vector fl = sha_org + vector(0,1,z-0.63);
	vector ar = al + lr; 
	vector br = bl + lr;
	vector cr = cl + lr;
	vector dr = dl + lr;
	vector er = el + lr;
	vector fr = fl + lr;
	r.area(el,dl,er,sha_col,sha_rot);
	r.area(al,el,ar,sha_col,sha_rot);
	r.area(al,bl,ar,sha_col,sha_rot);
	r.area(bl,cl,br,sha_col,sha_rot);
	r.area(dl,cl,dr,sha_col,sha_rot);
	r.area(al,bl,el,sha_col,sha_rot);
	r.area(ar,br,er,sha_col,sha_rot);
	r.area(dr,cr,fr,sha_col,sha_rot);
	r.area(dl,cl,fl,sha_col,sha_rot);
	r.triang(fl,dl,el,sha_col,sha_rot);
	r.triang(fr,dr,er,sha_col,sha_rot);
	r.line(al,ar,OUTLINE,sha_rot);
	r.line(bl,br,OUTLINE,sha_rot);
	r.line(cl,cr,OUTLINE,sha_rot);
	r.line(dl,dr,OUTLINE,sha_rot);
	r.line(el,er,OUTLINE,sha_rot);
	r.line(al,bl,OUTLINE,sha_rot);
	r.line(ar,br,OUTLINE,sha_rot);
	r.line(bl,cl,OUTLINE,sha_rot);
	r.line(br,cr,OUTLINE,sha_rot);
	r.line(cr,dr,OUTLINE,sha_rot);
	r.line(cl,dl,OUTLINE,sha_rot);
	r.line(dl,el,OUTLINE,sha_rot);
	r.line(dr,er,OUTLINE,sha_rot);
	r.line(el,al,OUTLINE,sha_rot);
	r.line(er,ar,OUTLINE,sha_rot);
	}

void roofbody::draw(room& r)
	{
	double x = sha_siz.x();
	double y = sha_siz.y();
	double z = sha_siz.z();
	vector lr(x,0,0);
	vector al = sha_org;
	vector bl = sha_org + vector(0,0,z);
	vector cl = sha_org + vector(0,y,0);
	vector dl = sha_org + vector(0,1,z);
	vector el = sha_org + vector(0,y,0.63);
	vector fl = sha_org + vector(0,0,0.63);
	vector gl = sha_org + vector(0,1,0.63);
	vector ar = al + lr; 
	vector br = bl + lr;
	vector cr = cl + lr;
	vector dr = dl + lr;
	vector er = el + lr;
	vector fr = fl + lr;
	vector gr = gl + lr;
	r.area(al,bl,ar,sha_col,sha_rot);
	r.area(bl,dl,br,sha_col,sha_rot);
	r.area(dl,el,dr,sha_col,sha_rot);
	r.area(cl,el,cr,sha_col,sha_rot);
	r.area(bl,al,dl,sha_col,sha_rot);
	r.area(br,ar,dr,sha_col,sha_rot);
	r.area(er,cr,gr,sha_col,sha_rot);
	r.area(el,cl,gl,sha_col,sha_rot);
	r.triang(gl,dl,el,sha_col,sha_rot);
	r.triang(gr,dr,er,sha_col,sha_rot);
	r.line(al,ar,OUTLINE,sha_rot);
	r.line(bl,br,OUTLINE,sha_rot);
	r.line(cl,cr,OUTLINE,sha_rot);
	r.line(dl,dr,OUTLINE,sha_rot);
	r.line(el,er,OUTLINE,sha_rot);
	r.line(al,bl,OUTLINE,sha_rot);
	r.line(ar,br,OUTLINE,sha_rot);
	r.line(al,cl,OUTLINE,sha_rot);
	r.line(ar,cr,OUTLINE,sha_rot);
	r.line(br,dr,OUTLINE,sha_rot);
	r.line(bl,dl,OUTLINE,sha_rot);
	r.line(dl,el,OUTLINE,sha_rot);
	r.line(dr,er,OUTLINE,sha_rot);
	r.line(cl,el,OUTLINE,sha_rot);
	r.line(cr,er,OUTLINE,sha_rot);
	}

void rooftop::draw(room& r)
	{
	double rad = 0.1;
	double hcyl = sha_siz.z()*0.4 - rad;
	double hyp = hypot(sha_siz.y()*0.5,hcyl);
	double alpha = asin(rad/hyp) + acos(0.5*sha_siz.y()/hyp);
	alpha *= 180*M_1_PI;
	vector x(sha_siz.x(),0,0);
	vector y(0,sha_siz.y(),0);
	vector z1(0,0,0.63);
	vector al(sha_org);
	vector bl(sha_org+y);
	vector cl(al+z1);
	vector dl(bl+z1);
	vector el(sha_org+y*0.5+vector(0,0,hcyl*2.5));
	vector fl(el+circularYZ(-alpha)*rad);	
	vector gl(el+circularYZ(+alpha)*rad);	
	vector ar(al+x);
	vector br(bl+x);
	vector cr(cl+x);
	vector dr(dl+x);
	vector er(el+x);
	vector fr(fl+x);
	vector gr(gl+x);
	r.cylsector(el,x,gl-el,2.0*alpha,sha_col,sha_rot);
	r.area(cl,cr,fl,sha_col,sha_rot);
	r.area(dl,dr,gl,sha_col,sha_rot);
	r.area(al,bl,cl,sha_col,sha_rot);
	r.area(ar,br,cr,sha_col,sha_rot);
	r.area(al,ar,cl,sha_col,sha_rot);
	r.area(bl,br,dl,sha_col,sha_rot);
	r.triang(el,fl,cl,sha_col,sha_rot);
	r.triang(el,cl,dl,sha_col,sha_rot);
	r.triang(el,dl,gl,sha_col,sha_rot);
	r.triang(er,fr,cr,sha_col,sha_rot);
	r.triang(er,cr,dr,sha_col,sha_rot);
	r.triang(er,dr,gr,sha_col,sha_rot);
	r.sector(el,x,gl-el,2*alpha,sha_col,sha_rot);
	r.sector(er,x,gr-er,2*alpha,sha_col,sha_rot);
	r.arc(el,x,gl-el,2*alpha,OUTLINE,sha_rot);
	r.arc(er,x,gr-er,2*alpha,OUTLINE,sha_rot);
	r.line(al,bl,OUTLINE,sha_rot);
	r.line(ar,br,OUTLINE,sha_rot);
	r.line(al,ar,OUTLINE,sha_rot);
	r.line(bl,br,OUTLINE,sha_rot);
	r.line(cl,cr,OUTLINE,sha_rot);
	r.line(dl,dr,OUTLINE,sha_rot);
	r.line(al,cl,OUTLINE,sha_rot);
	r.line(ar,cr,OUTLINE,sha_rot);
	r.line(bl,dl,OUTLINE,sha_rot);
	r.line(br,dr,OUTLINE,sha_rot);
	r.line(cl,fl,OUTLINE,sha_rot);
	r.line(cr,fr,OUTLINE,sha_rot);
	r.line(dl,gl,OUTLINE,sha_rot);
	r.line(dr,gr,OUTLINE,sha_rot);
	}

void roofcornerbody::draw(room& r)
	{
	vector x0(sha_org.x(),0,0);
	vector x1(sha_org.x()+1,0,0);
	vector x2(sha_org.x()+sha_siz.x(),0,0);
	vector y0(0,sha_org.y(),0);
	vector y1(0,sha_org.y()+1,0);
	vector y2(0,sha_org.y()+sha_siz.y(),0);
	vector z0(0,0,sha_org.z());
	vector z1(0,0,sha_org.z()+0.63);
	vector z2(0,0,sha_org.z()+sha_siz.z());
	vector a(x0+y0+z0);
	vector b(x2+y0+z0);
	vector c(x2+y2+z0);
	vector d(x0+y2+z0);
	vector e(x1+y0+z1);
	vector f(x2+y0+z1);
	vector g(x2+y2+z1);
	vector h(x0+y2+z1);
	vector i(x0+y1+z1);
	vector j(x0+y0+z2);
	vector k(x1+y0+z2);
	vector l(x1+y1+z2);
	vector m(x0+y1+z2);
	vector n(x2+y1+z1);
	vector o(x1+y2+z1);
	r.area(b,a,f,sha_col,sha_rot);
	r.area(b,c,f,sha_col,sha_rot);
	r.area(c,g,d,sha_col,sha_rot);
	r.area(d,h,a,sha_col,sha_rot);
	r.area(k,j,e,sha_col,sha_rot);
	r.area(m,i,j,sha_col,sha_rot);
	r.area(j,k,m,sha_col,sha_rot);
	r.area(k,f,l,sha_col,sha_rot);
	r.area(m,l,h,sha_col,sha_rot);
	r.triang(e,f,k,sha_col,sha_rot);
	r.triang(i,h,m,sha_col,sha_rot);
	r.triang(n,l,g,sha_col,sha_rot);
	r.triang(o,l,g,sha_col,sha_rot);
	r.line(a,b,OUTLINE,sha_rot);
	r.line(b,c,OUTLINE,sha_rot);
	r.line(c,d,OUTLINE,sha_rot);
	r.line(d,a,OUTLINE,sha_rot);
	r.line(b,f,OUTLINE,sha_rot);
	r.line(c,g,OUTLINE,sha_rot);
	r.line(d,h,OUTLINE,sha_rot);
	r.line(f,g,OUTLINE,sha_rot);
	r.line(g,h,OUTLINE,sha_rot);
	r.line(a,j,OUTLINE,sha_rot);
	r.line(f,k,OUTLINE,sha_rot);
	r.line(g,l,OUTLINE,sha_rot);
	r.line(h,m,OUTLINE,sha_rot);
	r.line(j,k,OUTLINE,sha_rot);
	r.line(k,l,OUTLINE,sha_rot);
	r.line(l,m,OUTLINE,sha_rot);
	r.line(m,j,OUTLINE,sha_rot);
	}

void cornerbody::draw(room& r)
	{
	vector x0(sha_org.x(),0,0);
	vector x1(sha_org.x()+1,0,0);
	vector x2(sha_org.x()+sha_siz.x(),0,0);
	vector y0(0,sha_org.y(),0);
	vector y1(0,sha_org.y()+1,0);
	vector y2(0,sha_org.y()+sha_siz.y(),0);
	vector z0(0,0,sha_org.z());
	vector z1(0,0,sha_org.z()+sha_siz.z());
	vector a(x0+y0+z0);
	vector b(x2+y0+z0);
	vector c(x2+y1+z0);
	vector d(x1+y1+z0);
	vector e(x1+y2+z0);
	vector f(x0+y2+z0);
	vector g(x0+y0+z1);
	vector h(x2+y0+z1);
	vector i(x2+y1+z1);
	vector j(x1+y1+z1);
	vector k(x1+y2+z1);
	vector l(x0+y2+z1);
	r.area(a,b,g,sha_col,sha_rot);
	r.area(a,f,g,sha_col,sha_rot);
	r.area(d,c,j,sha_col,sha_rot);
	r.area(d,e,j,sha_col,sha_rot);
	r.area(b,c,h,sha_col,sha_rot);
	r.area(f,e,l,sha_col,sha_rot);
	r.area(h,g,i,sha_col,sha_rot);
	r.area(k,j,l,sha_col,sha_rot);
	r.line(a,b,OUTLINE,sha_rot);
	r.line(b,c,OUTLINE,sha_rot);
	r.line(c,d,OUTLINE,sha_rot);
	r.line(d,e,OUTLINE,sha_rot);
	r.line(e,f,OUTLINE,sha_rot);
	r.line(f,a,OUTLINE,sha_rot);
	r.line(g,h,OUTLINE,sha_rot);
	r.line(h,i,OUTLINE,sha_rot);
	r.line(i,j,OUTLINE,sha_rot);
	r.line(j,k,OUTLINE,sha_rot);
	r.line(k,l,OUTLINE,sha_rot);
	r.line(l,g,OUTLINE,sha_rot);
	r.line(a,g,OUTLINE,sha_rot);
	r.line(b,h,OUTLINE,sha_rot);
	r.line(c,i,OUTLINE,sha_rot);
	r.line(d,j,OUTLINE,sha_rot);
	r.line(e,k,OUTLINE,sha_rot);
	r.line(f,l,OUTLINE,sha_rot);
	}

void roundbody::draw(room& r)
	{
	vector d(0,0,1);
	vector o(sha_org+vector(0.5,0.5,0));
	double r1 = 0.4;
	double r2 = 0.5;
	r.circle(o,d,r1,OUTLINE,sha_rot);
	vector h = d*0.6;
	r.cyl(o,h,r1,sha_col,sha_rot);
	o += h;
	r.circle(o,d,r1,OUTLINE,sha_rot);
	r.circle(o,d,r2,OUTLINE,sha_rot);
	r.ring(o,d,r1,r2,sha_col,sha_rot);
	h = d*(sha_siz.z()-0.6);
	r.cyl(o,h,r2,sha_col,sha_rot);
	o += h;
	r.circle(o,d,r2,OUTLINE,sha_rot);
	r1 = 0.3;
	r.ring(o,d,r1,r2,sha_col,sha_rot);
	}

void curved1body::draw(room& r)
	{
	vector x0(sha_siz.x(),0,0);
	vector y0(0,sha_siz.y(),0);
	vector y1(0,sha_siz.y()-1,0);
	vector z0(0,0,sha_siz.z());
	vector z1(0,0,sha_siz.z()-2.5);
	vector al(sha_org);
	vector bl(sha_org+z0);
	vector cl(sha_org+y0);
	vector dl(sha_org+y1+z0);
	vector el(sha_org+y0+z1);
	vector fl(sha_org+y1+z1);
	vector gl(sha_org+y1);
	vector ar(al+x0);
	vector br(bl+x0);
	vector cr(cl+x0);
	vector dr(dl+x0);
	vector er(el+x0);
	vector fr(fl+x0);
	vector gr(gl+x0);
	r.area(bl,dl,br,sha_col,sha_rot);
	r.area(al,bl,ar,sha_col,sha_rot);
	r.area(al,bl,gl,sha_col,sha_rot);
	r.area(ar,br,gr,sha_col,sha_rot);
	r.area(gl,cl,fl,sha_col,sha_rot);
	r.area(gr,cr,fr,sha_col,sha_rot);
	r.area(cl,el,cr,sha_col,sha_rot);
	r.sector(fl,x0,el-fl,90,sha_col,sha_rot);
	r.sector(fr,x0,er-fr,90,sha_col,sha_rot);
	r.cylsector(fl,x0,el-fl,90,sha_col,sha_rot);
	r.line(al,cl,OUTLINE,sha_rot);
	r.line(ar,cr,OUTLINE,sha_rot);
	r.line(al,bl,OUTLINE,sha_rot);
	r.line(ar,br,OUTLINE,sha_rot);
	r.line(bl,dl,OUTLINE,sha_rot);
	r.line(br,dr,OUTLINE,sha_rot);
	r.line(al,ar,OUTLINE,sha_rot);
	r.line(bl,br,OUTLINE,sha_rot);
	r.line(cl,cr,OUTLINE,sha_rot);
	r.line(cl,el,OUTLINE,sha_rot);
	r.line(cr,er,OUTLINE,sha_rot);
	r.arc(fl,x0,el-fl,90,OUTLINE,sha_rot);
	r.arc(fr,x0,er-fr,90,OUTLINE,sha_rot);
	}

void curved2body::draw(room& r)
	{
	vector x0(sha_siz.x(),0,0);
	vector x1(-sha_siz.x(),0,0);
	vector y0(0,sha_siz.y(),0);
	vector y1(0,1,0);
	vector y2(0,sha_siz.y()-1,0);
	vector z0(0,0,sha_siz.z());
	vector z1(0,0,sha_siz.z()-2.5);
	vector al(sha_org);
	vector bl(sha_org+y1+z0);
	vector cl(sha_org+y0);
	vector dl(sha_org+y2+z0);
	vector el(sha_org+y0+z1);
	vector fl(sha_org+y2+z1);
	vector gl(sha_org+y1+z1);
	vector hl(sha_org+z1);
	vector ar(al+x0);
	vector br(bl+x0);
	vector cr(cl+x0);
	vector dr(dl+x0);
	vector er(el+x0);
	vector fr(fl+x0);
	vector gr(gl+x0);
	vector hr(hl+x0);
	r.area(bl,dl,br,sha_col,sha_rot);
	r.area(al,hl,ar,sha_col,sha_rot);
	r.area(gl,fl,bl,sha_col,sha_rot);
	r.area(gr,fr,br,sha_col,sha_rot);
	r.area(al,cl,hl,sha_col,sha_rot);
	r.area(ar,cr,hr,sha_col,sha_rot);
	r.area(cl,el,cr,sha_col,sha_rot);
	r.sector(gl,x1,hl-gl,90,sha_col,sha_rot);
	r.sector(gr,x1,hr-gr,90,sha_col,sha_rot);
	r.sector(fl,x0,el-fl,90,sha_col,sha_rot);
	r.sector(fr,x0,er-fr,90,sha_col,sha_rot);
	r.cylsector(gr,x1,hr-gr,90,sha_col,sha_rot);
	r.cylsector(fl,x0,el-fl,90,sha_col,sha_rot);
	r.line(al,cl,OUTLINE,sha_rot);
	r.line(ar,cr,OUTLINE,sha_rot);
	r.line(al,hl,OUTLINE,sha_rot);
	r.line(ar,hr,OUTLINE,sha_rot);
	r.line(bl,dl,OUTLINE,sha_rot);
	r.line(br,dr,OUTLINE,sha_rot);
	r.line(al,ar,OUTLINE,sha_rot);
	r.line(cl,cr,OUTLINE,sha_rot);
	r.line(cl,el,OUTLINE,sha_rot);
	r.line(cr,er,OUTLINE,sha_rot);
	r.arc(gl,x1,hl-gl,90,OUTLINE,sha_rot);
	r.arc(gr,x1,hr-gr,90,OUTLINE,sha_rot);
	r.arc(fl,x0,el-fl,90,OUTLINE,sha_rot);
	r.arc(fr,x0,er-fr,90,OUTLINE,sha_rot);
	}

void curved3::draw(room& r)
	{
	vector x0(sha_org.x(),0,0);
	vector xd(sha_siz.x(),0,0);
	vector y0(0,sha_org.y(),0);
	vector y1(y0+vector(0,1,0));
	vector y2(y0+vector(0,2,0));
	vector y3(y0+vector(0,3,0));
	vector y4(y0+vector(0,4,0));
	vector z0(0,0,sha_org.z());
	vector z1(z0+vector(0,0,0.5));
	vector z2(z0+vector(0,0,3));
	vector z3(z0+vector(0,0,3.5));
	vector al(x0+y0+z0);
	vector bl(x0+y1+z2);
	vector cl(x0+y4+z0);
	vector dl(x0+y3+z2);
	vector el(x0+y4+z1);
	vector fl(x0+y3+z1);
	vector gl(x0+y1+z1);
	vector hl(x0+y0+z1);
	vector il(x0+y3+z3);
	vector jl(x0+y1+z3);
	vector kl(x0+y2+z3);
	vector ar(al+xd);
	vector br(bl+xd);
	vector cr(cl+xd);
	vector dr(dl+xd);
	vector er(el+xd);
	vector fr(fl+xd);
	vector gr(gl+xd);
	vector hr(hl+xd);
	vector ir(il+xd);
	vector jr(jl+xd);
	vector kr(kl+xd);
	r.area(dl,il,dr,sha_col,sha_rot);
	r.area(bl,jl,br,sha_col,sha_rot);
	r.area(al,hl,ar,sha_col,sha_rot);
	r.area(gl,fl,jl,sha_col,sha_rot);
	r.area(gr,fr,jr,sha_col,sha_rot);
	r.area(al,cl,hl,sha_col,sha_rot);
	r.area(ar,cr,hr,sha_col,sha_rot);
	r.area(cl,el,cr,sha_col,sha_rot);
	r.sector(gl,xd,bl-gl,90,sha_col,sha_rot);
	r.sector(gr,xd,br-gr,90,sha_col,sha_rot);
	r.sector(fl,xd,el-fl,90,sha_col,sha_rot);
	r.sector(fr,xd,er-fr,90,sha_col,sha_rot);
	r.sector(kl,xd,il-kl,180,sha_col,sha_rot);
	r.sector(kr,xd,ir-kr,180,sha_col,sha_rot);
	r.cylsector(gl,xd,br-gr,90,sha_col,sha_rot);
	r.cylsector(fl,xd,el-fl,90,sha_col,sha_rot);
	r.cylsector(kl,xd,il-kl,180,sha_col,sha_rot);
	r.line(dl,il,OUTLINE,sha_rot);
	r.line(dr,ir,OUTLINE,sha_rot);
	r.line(bl,jl,OUTLINE,sha_rot);
	r.line(br,jr,OUTLINE,sha_rot);
	r.line(al,cl,OUTLINE,sha_rot);
	r.line(ar,cr,OUTLINE,sha_rot);
	r.line(al,hl,OUTLINE,sha_rot);
	r.line(ar,hr,OUTLINE,sha_rot);
	r.line(bl,br,OUTLINE,sha_rot);
	r.line(dl,dr,OUTLINE,sha_rot);
	r.line(al,ar,OUTLINE,sha_rot);
	r.line(cl,cr,OUTLINE,sha_rot);
	r.line(cl,el,OUTLINE,sha_rot);
	r.line(cr,er,OUTLINE,sha_rot);
	r.arc(kl,xd,il-kl,180,OUTLINE,sha_rot);
	r.arc(kr,xd,ir-kr,180,OUTLINE,sha_rot);
	r.arc(gl,xd,bl-gl,90,OUTLINE,sha_rot);
	r.arc(gr,xd,br-gr,90,OUTLINE,sha_rot);
	r.arc(fl,xd,el-fl,90,OUTLINE,sha_rot);
	r.arc(fr,xd,er-fr,90,OUTLINE,sha_rot);
	}

void curved4::draw(room& r)
	{
	vector x(sha_siz.x(),0,0);
	vector y(0,sha_siz.y(),0);
	vector h(0,0,sha_siz.z()-2.5*sha_siz.x()*0.5);
	vector a0(sha_org);
	vector b0(a0+x);
	vector c0(b0+y);
	vector d0(a0+y);
	vector a1(a0+h);
	vector b1(b0+h);
	vector c1(c0+h);
	vector d1(d0+h);
	vector m1((a1+b1)*0.5);
	vector m2(m1+y);
	vector rad(-0.5*x);
	r.area(a0,a1,b0,sha_col,sha_rot);
	r.area(b0,b1,c0,sha_col,sha_rot);
	r.area(c0,c1,d0,sha_col,sha_rot);
	r.area(d0,d1,a0,sha_col,sha_rot);
	r.sector(m1,y,rad,180,sha_col,sha_rot);
	r.sector(m2,y,rad,180,sha_col,sha_rot);
	r.cylsector(m1,y,rad,180,sha_col,sha_rot);
	r.line(a0,b0,OUTLINE,sha_rot);
	r.line(b0,c0,OUTLINE,sha_rot);
	r.line(c0,d0,OUTLINE,sha_rot);
	r.line(d0,a0,OUTLINE,sha_rot);
	r.line(a0,a1,OUTLINE,sha_rot);
	r.line(b0,b1,OUTLINE,sha_rot);
	r.line(c0,c1,OUTLINE,sha_rot);
	r.line(d0,d1,OUTLINE,sha_rot);
	r.arc(m1,y,a1-m1,180,OUTLINE,sha_rot);
	r.arc(m2,y,d1-m2,180,OUTLINE,sha_rot);
	}

void roundcorner1body::draw(room& r)
	{
	const double alpha = 36.5;
	double omega = alpha*DEGREE;
	vector z1(0,0,0.75);
	vector z2(0,0,sha_siz.z());
	vector a1(0,1,0);
	vector b1(0,sha_siz.y(),0);
	vector c1(1,0,0);
	vector d1(sha_siz.x(),0,0);
	vector r1(c1);
	vector e1(d1*cos(omega)+b1*sin(omega)+sha_org);
	omega = M_PI_2 - omega;
	vector f1(d1*cos(omega)+b1*sin(omega)+sha_org);
	a1 += sha_org;
	b1 += sha_org;
	c1 += sha_org;
	d1 += sha_org;
	vector a2(a1+z2);
	vector b2(b1+z2);
	vector c2(c1+z2);
	vector d2(d1+z2);
	vector e2(e1+z1);
	vector f2(f1+z1);
	vector r2(d1-sha_org);
	r.line(a1,b1,OUTLINE,sha_rot);
	r.line(a2,b2,OUTLINE,sha_rot);
	r.line(c1,d1,OUTLINE,sha_rot);
	r.line(c2,d2,OUTLINE,sha_rot);
	r.line(a1,a2,OUTLINE,sha_rot);
	r.line(b1,b2,OUTLINE,sha_rot);
	r.line(c1,c2,OUTLINE,sha_rot);
	r.line(d1,d2,OUTLINE,sha_rot);
	r.arc(sha_org+z2,z2,r1,90,OUTLINE,sha_rot);
	r.arc(sha_org,z2,r1,90,OUTLINE,sha_rot);
	r.arc(sha_org+z2,z2,r2,90,OUTLINE,sha_rot);
	r.arc(sha_org,z2,r2,alpha,OUTLINE,sha_rot);
	r.arc(sha_org,z2,b1-sha_org,-alpha,OUTLINE,sha_rot);
	r.line(e1,e2,OUTLINE,sha_rot);
	r.line(f1,f2,OUTLINE,sha_rot);
	r.arc(sha_org+z1,z1,e1-sha_org,90-2*alpha,OUTLINE,sha_rot);
	r.cylsector(sha_org,z2,r1,90,sha_col,sha_rot);
	r.cylsector(sha_org+z1,z2-z1,r2,90,sha_col,sha_rot);
	r.cylsector(sha_org,z1,r2,alpha,sha_col,sha_rot);
	r.cylsector(sha_org,z1,b1-sha_org,-alpha,sha_col,sha_rot);
	r.ringsector(sha_org+z2,z2,c1-sha_org,d1-sha_org,90,sha_col,sha_rot);
	r.area(a1,b1,a2,sha_col,sha_rot);
	r.area(c1,d1,c2,sha_col,sha_rot);
	}

void roundcorner2body::draw(room& r)
	{
	const double alpha = 18;
	const double beta  = 34.5;
	double omega;
	vector z1(0,0,0.75);
	vector z2(0,0,sha_siz.z());
	vector o1(sha_org);
	vector o2(o1+z2);
	vector o3(o1+z1);
	vector rx(sha_siz.x(),0,0);
	vector ry(0,sha_siz.y(),0);
	vector a1(vector(0,sha_siz.y(),0)+sha_org);
	vector b1(vector(sha_siz.x(),0,0)+sha_org);
	omega = alpha*DEGREE;
	vector c1(rx*cos(omega)+ry*sin(omega)+sha_org);
	omega = (45-0.5*beta)*DEGREE;
	vector d1(rx*cos(omega)+ry*sin(omega)+sha_org);
	omega = (45+0.5*beta)*DEGREE;
	vector e1(rx*cos(omega)+ry*sin(omega)+sha_org);
	omega = (90-alpha)*DEGREE;
	vector f1(rx*cos(omega)+ry*sin(omega)+sha_org);
	vector a2(a1+z2);
	vector b2(b1+z2);
	vector c2(c1+z1);
	vector d2(d1+z1);
	vector e2(e1+z1);
	vector f2(f1+z1);
	r.line(o1,a1,OUTLINE,sha_rot);
	r.line(o2,a2,OUTLINE,sha_rot);
	r.line(o1,b1,OUTLINE,sha_rot);
	r.line(o2,b2,OUTLINE,sha_rot);
	r.line(o1,o2,OUTLINE,sha_rot);
	r.line(a1,a2,OUTLINE,sha_rot);
	r.line(b1,b2,OUTLINE,sha_rot);
	r.arc(o2,z2,rx,90,OUTLINE,sha_rot);
	r.arc(o1,z2,rx,alpha,OUTLINE,sha_rot);
	r.arc(o1,z2,d1-o1,beta,OUTLINE,sha_rot);
	r.arc(o1,z2,ry,-alpha,OUTLINE,sha_rot);	
	r.line(c1,c2,OUTLINE,sha_rot);
	r.line(d1,d2,OUTLINE,sha_rot);
	r.line(e1,e2,OUTLINE,sha_rot);
	r.line(f1,f2,OUTLINE,sha_rot);
	omega = 0.5*(90-alpha-alpha-beta);
	r.arc(o3,z1,c1-o1,omega,OUTLINE,sha_rot);
	r.arc(o3,z1,e1-o1,omega,OUTLINE,sha_rot);
	r.cylsector(o3,z2-z1,rx,90,sha_col,sha_rot);
	r.cylsector(o1,z1,rx,alpha,sha_col,sha_rot);
	r.cylsector(o1,z1,d1-o1,beta,sha_col,sha_rot);
	r.cylsector(o1,z1,ry,-alpha,sha_col,sha_rot);
	r.sector(o2,z2,rx,90,sha_col,sha_rot);
	r.area(o1,a1,o2,sha_col,sha_rot);
	r.area(o1,b1,o2,sha_col,sha_rot);
	}

void navepin1::draw(room& r)
	{
	const double R1 = 0.15;
	const double R2 = 0.3;
	const double R3 = 0.37;
	vector org(sha_org);
	r.circle(org,sha_siz,R3,OUTLINE,sha_rot);
	r.cyl(org,sha_siz*0.1,R3,sha_col,sha_rot);
	org += sha_siz*0.1;
	r.circle(org,sha_siz,R3,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R2,R3,sha_col,sha_rot);
	r.circle(org,sha_siz,R2,OUTLINE,sha_rot);
	r.cyl(org,sha_siz*0.9,R2,sha_col,sha_rot);
	org = sha_org + sha_siz;
	r.circle(org,sha_siz,R2,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R1,R2,sha_col,sha_rot);
	r.disc(org,sha_siz,R1,sha_col.dark(),sha_rot);
	r.circle(org,sha_siz,R1,OUTLINE,sha_rot);
	}

void navepin2::draw(room& r)
	{
	const double R1 = 0.2;
	const double R2 = 0.25;
	vector org(sha_org);
	vector siz(sha_siz*0.75);
	r.disc(org,sha_siz,R2,sha_col,sha_rot);
	r.circle(org,sha_siz,R2,OUTLINE,sha_rot);
	r.cyl(org,siz*0.15,R2,sha_col,sha_rot);
	org += siz*0.15;
	r.circle(org,sha_siz,R2,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R1,R2,sha_col,sha_rot);
	r.circle(org,sha_siz,R1,OUTLINE,sha_rot);
	r.cyl(org,siz*0.85,R1,sha_col,sha_rot);
	org = sha_org + siz;
	r.circle(org,sha_siz,R1,OUTLINE,sha_rot);
	r.disc(org,sha_siz,R1,sha_col,sha_rot);
	}

void rimbody::draw(room& r)
	{
	vector org(sha_org);
	color DARK(sha_col.dark());
	r.circle(org,sha_siz,0.38,OUTLINE,sha_rot);
	r.circle(org,sha_siz,0.50,OUTLINE,sha_rot);
	r.ring(org,sha_siz,0.38,0.50,sha_col,sha_rot);
	r.circle(org,sha_siz,0.74,OUTLINE,sha_rot);
	r.circle(org,sha_siz,1.05,OUTLINE,sha_rot);
	r.ring(org,sha_siz,0.74,1.05,sha_col,sha_rot);
	r.cyl(org,sha_siz*0.1,0.38,DARK,sha_rot);
	r.cyl(org,sha_siz*0.9,0.50,sha_col,sha_rot);
	r.cyl(org,sha_siz*0.9,0.74,DARK,sha_rot);
	r.cyl(org,sha_siz*0.2,1.05,sha_col,sha_rot);
	org += sha_siz*0.1;
	r.circle(org,sha_siz,0.38,OUTLINE,sha_rot);
	r.ring(org,sha_siz,0.3,0.38,sha_col,sha_rot);
	r.circle(org,sha_siz,0.3,OUTLINE,sha_rot);
	r.cyl(org,sha_siz*0.8,0.3,DARK,sha_rot);
	org += sha_siz*0.1;
	r.circle(org,sha_siz,1.05,OUTLINE,sha_rot);
	r.ring(org,sha_siz,0.9,1.05,sha_col,sha_rot);
	r.circle(org,sha_siz,0.9,OUTLINE,sha_rot);
	r.cyl(org,sha_siz*0.6,0.9,sha_col,sha_rot);
	org += sha_siz*0.6;
	r.ring(org,sha_siz,0.9,1.05,sha_col,sha_rot);
	r.circle(org,sha_siz,1.05,OUTLINE,sha_rot);
	r.cyl(org,sha_siz*0.2,1.05,sha_col,sha_rot);
	r.circle(org,sha_siz,0.9,OUTLINE,sha_rot);
	org += sha_siz*0.1;
	r.circle(org,sha_siz,0.3,OUTLINE,sha_rot);
	r.ring(org,sha_siz,0.3,0.38,sha_col,sha_rot);
	r.circle(org,sha_siz,0.38,OUTLINE,sha_rot);
	r.cyl(org,sha_siz*0.1,0.38,DARK,sha_rot);
	r.circle(org,sha_siz,0.5,OUTLINE,sha_rot);
	r.circle(org,sha_siz,0.74,OUTLINE,sha_rot);
	r.ring(org,sha_siz,0.5,0.74,sha_col,sha_rot);
	org = sha_org+sha_siz;
	r.circle(org,sha_siz,1.05,OUTLINE,sha_rot);
	r.ring(org,sha_siz,0.38,1.05,sha_col,sha_rot);
	r.circle(org,sha_siz,0.38,OUTLINE,sha_rot);
	}

rimsmall::rimsmall(param& p)
	{
	sha_org = p.org();
	sha_siz = p.siz();
	sha_col = p.col();
	sha_rot = p.rot();
	if(p.extrastr()!="") rimsmall_arc = p.extraint();
	else rimsmall_arc = 90.0*(double)random()/(double)RAND_MAX;
	}

void rimsmall::draw(room& r)
	{
	const double R1 = 0.21;
	const double R2 = 0.38;
	const double R3 = 0.51;
	const double R4 = 0.4;
	const double R5 = 0.2;
	double arc0,arc;
	arc0 = rimsmall_arc;
	color DARK(sha_col.dark());
	vector siz(sha_siz);
	vector org0(sha_org+sha_siz.unit()*0.12);
	vector org(org0);
	r.circle(org,sha_siz,R3,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R1,R3,sha_col,sha_rot);
	r.circle(org,siz,R1,OUTLINE,sha_rot);
	r.cyl(org,siz*0.85,R1,DARK,sha_rot);
	r.circle(org,siz,R1,OUTLINE,sha_rot);
	org = org0 + siz*0.85;
	r.disc(org,sha_siz,R1,sha_col,sha_rot);
	org = org0;
	r.cyl(org,siz*0.3,R3,sha_col,sha_rot);
	org += siz*0.3;
	r.circle(org,sha_siz,R3,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R2,R3,sha_col,sha_rot);
	r.circle(org,sha_siz,R2,OUTLINE,sha_rot);
	r.cyl(org,siz*0.4,R2,sha_col,sha_rot);
	org += siz*0.4;
	r.circle(org,sha_siz,R3,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R2,R3,sha_col,sha_rot);
	r.circle(org,sha_siz,R2,OUTLINE,sha_rot);
	r.cyl(org,siz*0.3,R3,sha_col,sha_rot);
	org += siz*0.3;
	r.circle(org,sha_siz,R3,OUTLINE,sha_rot);
	org = org0+siz;
	r.ring(org,sha_siz,R4,R3,sha_col,sha_rot);
	r.disc(org,sha_siz,R5,sha_col,sha_rot);
	org -= 0.1*siz;
	r.ring(org,sha_siz,R5,R4,sha_col,sha_rot);
	// spokes:
	org = org0 + siz;
	vector h(0.1*siz);
	vector P1,P2,P3,P4,P5,P6,P7,P8;		// inner points
	vector Q1,Q2,Q3,Q4,Q5,Q6,Q7,Q8;		// outer points
	arc = arc0+22;	P1 = org + R5*circularXZ(arc); 
	arc += 46;	P2 = org + R5*circularXZ(arc); 
	arc += 44;	P3 = org + R5*circularXZ(arc);
	arc += 46;	P4 = org + R5*circularXZ(arc);
	arc += 44;	P5 = org + R5*circularXZ(arc);
	arc += 46;	P6 = org + R5*circularXZ(arc);
	arc += 44;	P7 = org + R5*circularXZ(arc);
	arc += 46;	P8 = org + R5*circularXZ(arc);
	arc = arc0+11;	Q1 = org + R4*circularXZ(arc);
	arc += 68;	Q2 = org + R4*circularXZ(arc);
	arc += 22;	Q3 = org + R4*circularXZ(arc);
	arc += 68;	Q4 = org + R4*circularXZ(arc);
	arc += 22;	Q5 = org + R4*circularXZ(arc);
	arc += 68;	Q6 = org + R4*circularXZ(arc);
	arc += 22;	Q7 = org + R4*circularXZ(arc);
	arc += 68;	Q8 = org + R4*circularXZ(arc);
	r.triang(P1,Q1,P8,sha_col,sha_rot);
	r.triang(Q8,Q1,P8,sha_col,sha_rot);
	r.triang(P3,Q3,P2,sha_col,sha_rot);
	r.triang(Q2,Q3,P2,sha_col,sha_rot);
	r.triang(P5,Q5,P4,sha_col,sha_rot);
	r.triang(Q4,Q5,P4,sha_col,sha_rot);
	r.triang(P7,Q7,P6,sha_col,sha_rot);
	r.triang(Q6,Q7,P6,sha_col,sha_rot);
	vector orgc(org-h);
	r.arc(orgc,h,P1-org,46,OUTLINE,sha_rot);
	r.arc(orgc,h,P3-org,46,OUTLINE,sha_rot);
	r.arc(orgc,h,P5-org,46,OUTLINE,sha_rot);
	r.arc(orgc,h,P7-org,46,OUTLINE,sha_rot);
	r.arc(orgc,h,Q1-org,68,OUTLINE,sha_rot);
	r.arc(orgc,h,Q3-org,68,OUTLINE,sha_rot);
	r.arc(orgc,h,Q5-org,68,OUTLINE,sha_rot);
	r.arc(orgc,h,Q7-org,68,OUTLINE,sha_rot);
	r.arc(org,h,P1-org,46,OUTLINE,sha_rot);
	r.arc(org,h,P3-org,46,OUTLINE,sha_rot);
	r.arc(org,h,P5-org,46,OUTLINE,sha_rot);
	r.arc(org,h,P7-org,46,OUTLINE,sha_rot);
	r.arc(org,h,Q1-org,68,OUTLINE,sha_rot);
	r.arc(org,h,Q3-org,68,OUTLINE,sha_rot);
	r.arc(org,h,Q5-org,68,OUTLINE,sha_rot);
	r.arc(org,h,Q7-org,68,OUTLINE,sha_rot);
	r.cylsector(orgc,h,P1-org,46,DARK,sha_rot);
	r.cylsector(orgc,h,P3-org,46,DARK,sha_rot);
	r.cylsector(orgc,h,P5-org,46,DARK,sha_rot);
	r.cylsector(orgc,h,P7-org,46,DARK,sha_rot);
	r.cylsector(orgc,h,Q1-org,68,DARK,sha_rot);
	r.cylsector(orgc,h,Q3-org,68,DARK,sha_rot);
	r.cylsector(orgc,h,Q5-org,68,DARK,sha_rot);
	r.cylsector(orgc,h,Q7-org,68,DARK,sha_rot);
	r.line(P1,P1-h,OUTLINE,sha_rot);
	r.line(P2,P2-h,OUTLINE,sha_rot);
	r.line(P3,P3-h,OUTLINE,sha_rot);
	r.line(P4,P4-h,OUTLINE,sha_rot);
	r.line(P5,P5-h,OUTLINE,sha_rot);
	r.line(P6,P6-h,OUTLINE,sha_rot);
	r.line(P7,P7-h,OUTLINE,sha_rot);
	r.line(P8,P8-h,OUTLINE,sha_rot);
	r.line(Q1,Q1-h,OUTLINE,sha_rot);
	r.line(Q2,Q2-h,OUTLINE,sha_rot);
	r.line(Q3,Q3-h,OUTLINE,sha_rot);
	r.line(Q4,Q4-h,OUTLINE,sha_rot);
	r.line(Q5,Q5-h,OUTLINE,sha_rot);
	r.line(Q6,Q6-h,OUTLINE,sha_rot);
	r.line(Q7,Q7-h,OUTLINE,sha_rot);
	r.line(Q8,Q8-h,OUTLINE,sha_rot);
	r.line(P1,Q1,OUTLINE,sha_rot);
	r.line(P2,Q2,OUTLINE,sha_rot);
	r.line(P3,Q3,OUTLINE,sha_rot);
	r.line(P4,Q4,OUTLINE,sha_rot);
	r.line(P5,Q5,OUTLINE,sha_rot);
	r.line(P6,Q6,OUTLINE,sha_rot);
	r.line(P7,Q7,OUTLINE,sha_rot);
	r.line(P8,Q8,OUTLINE,sha_rot);
	r.line(P1-h,Q1-h,OUTLINE,sha_rot);
	r.line(P2-h,Q2-h,OUTLINE,sha_rot);
	r.line(P3-h,Q3-h,OUTLINE,sha_rot);
	r.line(P4-h,Q4-h,OUTLINE,sha_rot);
	r.line(P5-h,Q5-h,OUTLINE,sha_rot);
	r.line(P6-h,Q6-h,OUTLINE,sha_rot);
	r.line(P7-h,Q7-h,OUTLINE,sha_rot);
	r.line(P8-h,Q8-h,OUTLINE,sha_rot);
	r.area(P1,Q1,P1-h,DARK,sha_rot);
	r.area(P2,Q2,P2-h,DARK,sha_rot);
	r.area(P3,Q3,P3-h,DARK,sha_rot);
	r.area(P4,Q4,P4-h,DARK,sha_rot);
	r.area(P5,Q5,P5-h,DARK,sha_rot);
	r.area(P6,Q6,P6-h,DARK,sha_rot);
	r.area(P7,Q7,P7-h,DARK,sha_rot);
	r.area(P8,Q8,P8-h,DARK,sha_rot);
	}

void tyre::draw(room& r)
	{
	double R1,R2,R3;
	int NIPS;
	color DARK(sha_col.dark());
	vector org0(sha_org);
	double sizval = sha_siz.absvalue(); 
	if((sizval>0.9) && (sizval<1.1)) 	// |d|==1 normal size
		{
		R1 = 1.05;
		R2 = 1.22;
		R3 = 1.53;
		NIPS = 2*22;
		}
	else if(sizval < 0.9)		// |d|==0.75 small size
		{
		R1 = 0.51;
		R2 = 0.76;
		R3 = 0.9;
		NIPS = 2*16;
		org0 += sha_siz.unit()*0.12;
		}
	else return;
	const double SF = R2/R3;
	const double W = 2.0*M_PI/(int)NIPS;
	const double WRAND = W*(double)random()/(double)RAND_MAX;
	vector org(org0);
	r.circle(org,sha_siz,R1,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R1,R2,sha_col,sha_rot);
	r.cyl(org,sha_siz,R1,DARK,sha_rot);
	r.cyl(org,sha_siz,R2,sha_col,sha_rot);
	org += sha_siz;
	r.circle(org,sha_siz,R1,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R1,R2,sha_col,sha_rot);
	org = org0 + sha_siz*0.5;
	r.circle(org,sha_siz,R2,OUTLINE,sha_rot);
	r.circle(org,sha_siz,R3,OUTLINE,sha_rot);
	r.ring(org,sha_siz,R2,R3,sha_col,sha_rot);
	vector vz(sha_siz*0.5);
	vector v0(R3*sin(WRAND),0,2.5*R3*cos(WRAND));
	vector v2(v0*SF);
	for(int i=1;i<=NIPS;i++)
		{
		vector v1(R3*sin(W*i+WRAND),0,2.5*R3*cos(W*i+WRAND));
		vector v3(v1*SF);
		r.line(org+v1-vz,org+v1+vz,OUTLINE,sha_rot);
		r.line(org+v3-vz,org+v3+vz,OUTLINE,sha_rot);
		r.line(org+v0+vz,org+v1+vz,OUTLINE,sha_rot);
		r.line(org+v1+vz,org+v3+vz,OUTLINE,sha_rot);
		r.line(org+v0+vz,org+v2+vz,OUTLINE,sha_rot);
		r.line(org+v2-vz,org+v3-vz,OUTLINE,sha_rot);
		r.line(org+v1,org+v3,OUTLINE,sha_rot);
		r.area(org+v1,org+v1+vz,org+v0,sha_col,sha_rot);
		r.area(org+v1,org+v1+vz,org+v3,sha_col,sha_rot);
		r.area(org+v0,org+v0+vz,org+v2,sha_col,sha_rot);
		r.triang(org+vz+v1,org+vz+v0,org+vz+v2,sha_col,sha_rot);
		r.triang(org+vz+v1,org+vz+v2,org+vz+v3,sha_col,sha_rot);
		v0 = v1;
		v2 = v3;
		vz *= -1;
		}
	}

void doorleftbody::draw(room& r)
	{
	vector cylorg(sha_org+vector(0.5,0.5,0));
	vector a(sha_org+vector(1,0.5,0));
	vector b(sha_org+vector(0.5,1,0));
	vector c(sha_org+vector(1,1,0));
	vector d(sha_org+vector(1,sha_siz.y(),0));
	vector e(sha_org+vector(0.8,sha_siz.y(),0));
	vector f(sha_org+vector(0.8,1,0));
	vector z(0,0,sha_siz.z());
	r.cylsector(cylorg,z,b-cylorg,270,sha_col,sha_rot);
	r.sector(cylorg+z,z,b-cylorg,270,sha_col,sha_rot);
	r.arc(cylorg,z,b-cylorg,270,OUTLINE,sha_rot);
	r.arc(cylorg+z,z,b-cylorg,270,OUTLINE,sha_rot);
	r.area(c+z,a+z,b+z,sha_col,sha_rot);
	r.area(a,d,a+z,sha_col,sha_rot);
	r.area(b,f,b+z,sha_col,sha_rot);
	r.area(d,e,d+z,sha_col,sha_rot);
	r.area(e,f,e+z,sha_col,sha_rot);
	r.area(e+z,d+z,f+z,sha_col,sha_rot);
	r.line(a,d,OUTLINE,sha_rot);
	r.line(a+z,d+z,OUTLINE,sha_rot);
	r.line(d,e,OUTLINE,sha_rot);
	r.line(d+z,e+z,OUTLINE,sha_rot);
	r.line(e,f,OUTLINE,sha_rot);
	r.line(e+z,f+z,OUTLINE,sha_rot);
	r.line(b,f,OUTLINE,sha_rot);
	r.line(b+z,f+z,OUTLINE,sha_rot);
	r.line(d,d+z,OUTLINE,sha_rot);
	r.line(e,e+z,OUTLINE,sha_rot);
	r.line(f,f+z,OUTLINE,sha_rot);
	// Handle:
	color DARK(sha_col.dark());
	a = sha_org + vector(1,sha_siz.y()-0.5,0.5*sha_siz.z()+0.8);
	b = a - vector(0,0,1.6);
	c = b - vector(0,0.17,0);
	d = a - vector(0,0.17,0);
	vector hx(0.3,0,0);
	r.area(a,b,a+hx,DARK,sha_rot);
	r.area(b,c,b+hx,DARK,sha_rot);
	r.area(c,d,c+hx,DARK,sha_rot);
	r.area(d,a,d+hx,DARK,sha_rot);
	r.area(a+hx,b+hx,d+hx,sha_col,sha_rot);
	r.line(a,b,OUTLINE,sha_rot);
	r.line(b,c,OUTLINE,sha_rot);
	r.line(c,d,OUTLINE,sha_rot);
	r.line(d,a,OUTLINE,sha_rot);
	r.line(a+hx,b+hx,OUTLINE,sha_rot);
	r.line(b+hx,c+hx,OUTLINE,sha_rot);
	r.line(c+hx,d+hx,OUTLINE,sha_rot);
	r.line(d+hx,a+hx,OUTLINE,sha_rot);
	r.line(a,a+hx,OUTLINE,sha_rot);
	r.line(b,b+hx,OUTLINE,sha_rot);
	r.line(c,c+hx,OUTLINE,sha_rot);
	r.line(d,d+hx,OUTLINE,sha_rot);
	}

void doorrightbody::draw(room& r)
	{
	vector cylorg(sha_org+vector(0.5,0.5,0));
	vector a(sha_org+vector(0,0.5,0));
	vector b(sha_org+vector(0.5,1,0));
	vector c(sha_org+vector(0,1,0));
	vector d(sha_org+vector(0,sha_siz.y(),0));
	vector e(sha_org+vector(0.2,sha_siz.y(),0));
	vector f(sha_org+vector(0.2,1,0));
	vector z(0,0,sha_siz.z());
	r.cylsector(cylorg,z,a-cylorg,270,sha_col,sha_rot);
	r.sector(cylorg+z,z,a-cylorg,270,sha_col,sha_rot);
	r.arc(cylorg,z,a-cylorg,270,OUTLINE,sha_rot);
	r.arc(cylorg+z,z,a-cylorg,270,OUTLINE,sha_rot);
	r.area(c+z,a+z,b+z,sha_col,sha_rot);
	r.area(a,d,a+z,sha_col,sha_rot);
	r.area(b,f,b+z,sha_col,sha_rot);
	r.area(d,e,d+z,sha_col,sha_rot);
	r.area(e,f,e+z,sha_col,sha_rot);
	r.area(e+z,d+z,f+z,sha_col,sha_rot);
	r.line(a,d,OUTLINE,sha_rot);
	r.line(a+z,d+z,OUTLINE,sha_rot);
	r.line(d,e,OUTLINE,sha_rot);
	r.line(d+z,e+z,OUTLINE,sha_rot);
	r.line(e,f,OUTLINE,sha_rot);
	r.line(e+z,f+z,OUTLINE,sha_rot);
	r.line(b,f,OUTLINE,sha_rot);
	r.line(b+z,f+z,OUTLINE,sha_rot);
	r.line(d,d+z,OUTLINE,sha_rot);
	r.line(e,e+z,OUTLINE,sha_rot);
	r.line(f,f+z,OUTLINE,sha_rot);
	// Handle:
	color DARK(sha_col.dark());
	a = sha_org + vector(0,sha_siz.y()-0.5,0.5*sha_siz.z()+0.8);
	b = a - vector(0,0,1.6);
	c = b - vector(0,0.17,0);
	d = a - vector(0,0.17,0);
	vector hx(-0.3,0,0);
	r.area(a,b,a+hx,DARK,sha_rot);
	r.area(b,c,b+hx,DARK,sha_rot);
	r.area(c,d,c+hx,DARK,sha_rot);
	r.area(d,a,d+hx,DARK,sha_rot);
	r.area(a+hx,b+hx,d+hx,sha_col,sha_rot);
	r.line(a,b,OUTLINE,sha_rot);
	r.line(b,c,OUTLINE,sha_rot);
	r.line(c,d,OUTLINE,sha_rot);
	r.line(d,a,OUTLINE,sha_rot);
	r.line(a+hx,b+hx,OUTLINE,sha_rot);
	r.line(b+hx,c+hx,OUTLINE,sha_rot);
	r.line(c+hx,d+hx,OUTLINE,sha_rot);
	r.line(d+hx,a+hx,OUTLINE,sha_rot);
	r.line(a,a+hx,OUTLINE,sha_rot);
	r.line(b,b+hx,OUTLINE,sha_rot);
	r.line(c,c+hx,OUTLINE,sha_rot);
	r.line(d,d+hx,OUTLINE,sha_rot);
	}

void turndiskbody2::draw(room& r)
	{
	double rad = 0.5*sha_siz.x() - 0.125;
	vector h(0,0,sha_siz.z());
	vector org(vector(0.5*sha_siz.x(),0.5*sha_siz.y(),0)+sha_org);
	r.circle(org,h,rad,OUTLINE,sha_rot);
	r.cyl(org,h,rad,sha_col,sha_rot);
	org += h;
	r.disc(org,h,rad,sha_col,sha_rot);
	r.circle(org,h,rad,OUTLINE,sha_rot);
	}

void turndiskbody4::draw(room& r)
	{
	double r1 = 0.3;
	double r2 = 0.5*sha_siz.x();
	vector h(0,0,sha_siz.z()-1);
	vector org(vector(r2,0.5*sha_siz.y(),1)+sha_org);
	r.circle(org,h,r1,OUTLINE,sha_rot);
	r.circle(org,h,r2,OUTLINE,sha_rot);
	r.cyl(org,h,r1,sha_col,sha_rot);
	r.cyl(org,h,r2,sha_col,sha_rot);
	org += h;
	r.ring(org,h,r1,r2,sha_col,sha_rot);
	r.circle(org,h,r1,OUTLINE,sha_rot);
	r.circle(org,h,r2,OUTLINE,sha_rot);
	}

void windowbody::draw(room& r)
	{
	vector x(sha_siz.x(),0,0);
	vector y(0,sha_siz.y(),0);
	vector z(0,0,sha_siz.z());
	vector o(sha_org);
	vector al(sha_org);
	vector ar(al+x);
	vector bl(y+sha_org);
	vector br(bl+x);
	vector cl(0.45*ZUNIT+bl);
	vector cr(cl+x);
	vector dl(cl-0.125*YUNIT+0.75*ZUNIT);
	vector dr(dl+x);
	vector el(vector(0,sha_siz.y()-0.125,sha_siz.z()-0.35)+sha_org);
	vector er(el+x);
	vector fl(el+0.125*YUNIT);
	vector fr(fl+x);
	vector gl(sha_org+y+z);
	vector gr(gl+x);
	vector hl(sha_org+z);
	vector hr(hl+x);
	vector il(al+0.2*XUNIT);
	vector ir(ar-0.2*XUNIT);
	vector jl(il+0.7*ZUNIT);
	vector jr(ir+0.7*ZUNIT);
	vector kl(jl-0.05*XUNIT);
	vector kr(jr+0.05*XUNIT);
	vector ll(sha_org+z-0.35*ZUNIT+0.15*XUNIT);
	vector lr(sha_org+z-0.35*ZUNIT-0.15*XUNIT+x);
	r.line(bl,br,OUTLINE,sha_rot);
	r.line(al,bl,OUTLINE,sha_rot);
	r.line(ar,br,OUTLINE,sha_rot);
	r.line(cl,cr,OUTLINE,sha_rot);
	r.line(bl,cl,OUTLINE,sha_rot);
	r.line(br,cr,OUTLINE,sha_rot);
	r.line(dl,dr,OUTLINE,sha_rot);
	r.line(cl,dl,OUTLINE,sha_rot);
	r.line(cr,dr,OUTLINE,sha_rot);
	r.line(el,er,OUTLINE,sha_rot);
	r.line(dl,el,OUTLINE,sha_rot);
	r.line(dr,er,OUTLINE,sha_rot);
	r.line(fl,fr,OUTLINE,sha_rot);
	r.line(el,fl,OUTLINE,sha_rot);
	r.line(er,fr,OUTLINE,sha_rot);
	r.line(gl,gr,OUTLINE,sha_rot);
	r.line(fl,gl,OUTLINE,sha_rot);
	r.line(fr,gr,OUTLINE,sha_rot);
	r.line(hl,hr,OUTLINE,sha_rot);
	r.line(gl,hl,OUTLINE,sha_rot);
	r.line(gr,hr,OUTLINE,sha_rot);
	r.line(hr,ar,OUTLINE,sha_rot);
	r.line(hl,al,OUTLINE,sha_rot);
	r.line(al,il,OUTLINE,sha_rot);
	r.line(ar,ir,OUTLINE,sha_rot);
	r.line(il,il+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(ir,ir+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(il,jl,OUTLINE,sha_rot);
	r.line(ir,jr,OUTLINE,sha_rot);
	r.line(il+0.8*YUNIT,jl+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(ir+0.8*YUNIT,jr+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(jl,jl+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(jr,jr+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(jl,kl,OUTLINE,sha_rot);
	r.line(jr,kr,OUTLINE,sha_rot);
	r.line(jl+0.8*YUNIT,kl+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(jr+0.8*YUNIT,kr+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(kl,kl+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(kr,kr+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(kl,ll,OUTLINE,sha_rot);
	r.line(kr,lr,OUTLINE,sha_rot);
	r.line(ll,ll+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(lr,lr+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(kl+0.8*YUNIT,ll+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(kr+0.8*YUNIT,lr+0.8*YUNIT,OUTLINE,sha_rot);
	r.line(ll,lr,OUTLINE,sha_rot);
	r.line(ll+0.8*YUNIT,lr+0.8*YUNIT,OUTLINE,sha_rot);
	r.area(bl,cl,br,sha_col,sha_rot);
	r.area(bl,cl,bl-0.125*YUNIT,sha_col,sha_rot);
	r.area(br,cr,br-0.125*YUNIT,sha_col,sha_rot);
	r.triang(cl,dl,cl-0.125*YUNIT,sha_col,sha_rot);
	r.triang(cr,dr,cr-0.125*YUNIT,sha_col,sha_rot);
	r.area(cl,dl,cr,sha_col,sha_rot);
	r.area(el,fl,er,sha_col,sha_rot);
	r.area(fl,gl,fr,sha_col,sha_rot);
	r.area(fl,el,gl,sha_col,sha_rot);
	r.area(fr,er,gr,sha_col,sha_rot);
	r.area(gl,hl,gr,sha_col,sha_rot);
	r.area(al,bl-0.125*YUNIT,hl,sha_col,sha_rot);
	r.area(ar,br-0.125*YUNIT,hr,sha_col,sha_rot);
	r.area(jl,il,kl,sha_col,sha_rot);
	r.area(jr,ir,kr,sha_col,sha_rot);
	r.area(il,jl,il+0.8*YUNIT,sha_col,sha_rot);
	r.area(ir,jr,ir+0.8*YUNIT,sha_col,sha_rot);
	r.area(jl,kl,jl+0.8*YUNIT,sha_col,sha_rot);
	r.area(jr,kr,jr+0.8*YUNIT,sha_col,sha_rot);
	r.area(kl,ll,kl+0.8*YUNIT,sha_col,sha_rot);
	r.area(kr,lr,kr+0.8*YUNIT,sha_col,sha_rot);
	r.area(ll,lr,ll+0.35*ZUNIT,sha_col,sha_rot);
	r.area(hl,al,hl+0.15*XUNIT,sha_col,sha_rot);
	r.area(hr,ar,hr-0.15*XUNIT,sha_col,sha_rot);
	// Zinken:
	vector dx(0.2*XUNIT);
	vector y1(0.8*YUNIT);
	vector y2(0.4*YUNIT);
	vector z1(0.75*ZUNIT);
	vector tmp(sha_org+y1);
	r.line(tmp+dx,tmp+XUNIT-dx,OUTLINE,sha_rot);
	r.area(tmp+dx,tmp+XUNIT-dx,tmp+dx+z1,sha_col,sha_rot);
	for(int i=1;i<sha_siz.x();i++)
		{
		vector org(sha_org+i*XUNIT);
		vector alu(org-dx+y1);
		vector aru(org+dx+y1);
		vector blu(org-dx+y2);
		vector bru(org+dx+y2);
		vector cu(org+y2);
		vector alo(alu+z1);
		vector aro(aru+z1);
		vector blo(blu+z1);
		vector bro(bru+z1);
		vector co(cu+z1);
		r.line(alu,blu,OUTLINE,sha_rot);
		r.line(aru,bru,OUTLINE,sha_rot);
		r.line(alo,blo,OUTLINE,sha_rot);
		r.line(aro,bro,OUTLINE,sha_rot);
		r.line(alu,alo,OUTLINE,sha_rot);
		r.line(aru,aro,OUTLINE,sha_rot);
		r.line(alo,aro,OUTLINE,sha_rot);
		r.arc(cu,ZUNIT,dx,-180,OUTLINE,sha_rot);
		r.arc(co,ZUNIT,dx,-180,OUTLINE,sha_rot);
		r.area(alu,blu,alo,sha_col,sha_rot);
		r.area(aru,bru,aro,sha_col,sha_rot);
		r.area(alo,blo,aro,sha_col,sha_rot);
		r.cylsector(cu,z1,dx,-180,sha_col,sha_rot);
		r.sector(co,ZUNIT,dx,-180,sha_col,sha_rot);
		r.line(aru,aru+XUNIT-dx-dx,OUTLINE,sha_rot);
		r.area(aru,aru+XUNIT-dx-dx,aro,sha_col,sha_rot);
		}
	// Frontframe:
	vector ul(dl);
	vector ur(dr);
	vector ol(el);
	vector ro(er);
	vector delta(0.2*ZUNIT);
	r.area(ul,ur,ul+delta,sha_col,sha_rot);
	r.area(ol,ro,ol-delta,sha_col,sha_rot);
	ul += delta;
	ur += delta;
	ol -= delta;
	ro -= delta;
	delta = 0.15*XUNIT;
	r.area(ul,ol,ul+delta,sha_col,sha_rot);
	r.area(ur,ro,ur-delta,sha_col,sha_rot);
	ul += delta;
	ur -= delta;
	ol += delta;
	ro -= delta;
	r.line(ul,ur,OUTLINE,sha_rot);
	r.line(ur,ro,OUTLINE,sha_rot);
	r.line(ro,ol,OUTLINE,sha_rot);
	r.line(ol,ul,OUTLINE,sha_rot);
	delta = vector(0,o.y()+y.y()-ul.y(),0);
	r.line(ul,ul+delta,OUTLINE,sha_rot);
	r.line(ur,ur+delta,OUTLINE,sha_rot);
	r.line(ol,ol+delta,OUTLINE,sha_rot);
	r.line(ro,ro+delta,OUTLINE,sha_rot);
	r.area(ul,ur,ul+delta,sha_col,sha_rot);
	r.area(ur,ro,ur+delta,sha_col,sha_rot);
	r.area(ro,ol,ro+delta,sha_col,sha_rot);
	r.area(ol,ul,ol+delta,sha_col,sha_rot);
	ul += delta;
	ur += delta;
	ol += delta;
	ro += delta;
	r.line(ul,ur,OUTLINE,sha_rot);
	r.line(ur,ro,OUTLINE,sha_rot);
	r.line(ro,ol,OUTLINE,sha_rot);
	r.line(ol,ul,OUTLINE,sha_rot);
	delta = 0.35*ZUNIT;
	r.area(ol,ro,ol-delta,sha_col,sha_rot);
	r.area(ul,ur,ul+delta,sha_col,sha_rot);
	ol -= delta;
	ro -= delta;
	ul += delta;
	ur += delta;
	delta = 0.15*XUNIT;
	r.area(ul,ol,ul+delta,sha_col,sha_rot);
	r.area(ur,ro,ur-delta,sha_col,sha_rot);
	ol += delta;
	ro -= delta;
	ul += delta;
	ur -= delta;
	r.line(ul,ur,OUTLINE,sha_rot);
	r.line(ur,ro,OUTLINE,sha_rot);
	r.line(ro,ol,OUTLINE,sha_rot);
	r.line(ol,ul,OUTLINE,sha_rot);
	delta = 0.2*YUNIT;
	r.area(ul,ur,ul-delta,sha_col,sha_rot);
	r.area(ur,ro,ur-delta,sha_col,sha_rot);
	r.area(ro,ol,ro-delta,sha_col,sha_rot);
	r.area(ol,ul,ol-delta,sha_col,sha_rot);
	r.line(ul,ul-delta,OUTLINE,sha_rot);
	r.line(ur,ur-delta,OUTLINE,sha_rot);
	r.line(ro,ro-delta,OUTLINE,sha_rot);
	r.line(ol,ol-delta,OUTLINE,sha_rot);
	ol -= delta;
	ro -= delta;
	ul -= delta;
	ur -= delta;
	r.line(ul,ur,OUTLINE,sha_rot);
	r.line(ur,ro,OUTLINE,sha_rot);
	r.line(ro,ol,OUTLINE,sha_rot);
	r.line(ol,ul,OUTLINE,sha_rot);
	delta = vector(ul.x()-o.x()-0.15,0,0);
	r.area(ul,ol,ul-delta,sha_col,sha_rot);
	r.area(ur,ro,ur+delta,sha_col,sha_rot);
	ol -= delta;
	ro += delta;
	ul -= delta;
	ur += delta;
	delta = vector(0,0,o.z()+z.z()-ol.z()-0.35);
	r.area(ol,ro,ol+delta,sha_col,sha_rot);
	ol += delta;
	ro += delta;
	delta = vector(0.01,0,0);
	r.area(ol-delta,ro-delta,ul-delta-vector(0,0,0.7),TRANSPARENT,sha_rot);
	delta = vector(0,0,ul.z()-o.z()-0.75);
	r.area(ul,ur,ul-delta,sha_col,sha_rot);
	}

void eye::draw(room& r)
	{
	vector org(vector(0.5,0.01,1.25)+sha_org);
	vector dir(0,1,0);
	r.disc(org,dir,0.37,sha_col,sha_rot);
	r.ring(org,dir,0.37,0.38,OUTLINE,sha_rot);
	org -= vector(0,0,0.32);
	r.disc(org,dir,0.25,OUTLINE,sha_rot);
	}

void wedgeleftbody::draw(room& r)
	{
	vector x(sha_siz.x(),0,0);
	vector y(0,sha_siz.y(),0);
	vector z(0,0,sha_siz.z());
	vector z1(0,0,0.75);
	vector d(sha_org);
	vector a(d+x);
	vector b(a+y);
	vector c(b-XUNIT);
	r.line(a,b,OUTLINE,sha_rot);
	r.line(b,c,OUTLINE,sha_rot);
	r.line(d,a,OUTLINE,sha_rot);
	r.line(a,a+z,OUTLINE,sha_rot);
	r.line(b,b+z,OUTLINE,sha_rot);
	r.line(c,c+z,OUTLINE,sha_rot);
	r.line(d,d+z,OUTLINE,sha_rot);
	r.line(a+z,b+z,OUTLINE,sha_rot);
	r.line(b+z,c+z,OUTLINE,sha_rot);
	r.line(c+z,d+z,OUTLINE,sha_rot);
	r.line(d+z,a+z,OUTLINE,sha_rot);
	r.area(a,b,a+z,sha_col,sha_rot);
	r.area(b,c,b+z,sha_col,sha_rot);
	r.area(c+z1,d+z1,c+z,sha_col,sha_rot);
	r.area(d,a,d+z,sha_col,sha_rot);
	r.area(b+z,a+z,c+z,sha_col,sha_rot);
	r.triang(a+z-XUNIT,c+z,d+z,sha_col,sha_rot);
	vector l(c-d);
	vector l1(d+0.106*l);
	vector l2(d+0.284*l);
	vector l3(d+0.39*l);
	vector l4(d+0.61*l);
	r.line(d,l1,OUTLINE,sha_rot);
	r.line(l1,l1+z1,OUTLINE,sha_rot);
	r.line(l1+z1,l2+z1,OUTLINE,sha_rot);
	r.line(l2+z1,l2,OUTLINE,sha_rot);
	r.line(l2,l3,OUTLINE,sha_rot);
	r.line(l3,l3+z1,OUTLINE,sha_rot);
	r.line(l3+z1,l4+z1,OUTLINE,sha_rot);
	r.line(l4+z1,l4,OUTLINE,sha_rot);
	r.line(l4,c,OUTLINE,sha_rot);
	r.area(d,l1,d+z1,sha_col,sha_rot);
	r.area(l2,l3,l2+z1,sha_col,sha_rot);
	r.area(l4,c,l4+z1,sha_col,sha_rot);
	}

void wedgerightbody::draw(room& r)
	{
	vector x(sha_siz.x(),0,0);
	vector y(0,sha_siz.y(),0);
	vector z(0,0,sha_siz.z());
	vector z1(0,0,0.75);
	vector a(sha_org);
	vector b(a+y);
	vector c(b+XUNIT);
	vector d(a+x);
	r.line(a,b,OUTLINE,sha_rot);
	r.line(b,c,OUTLINE,sha_rot);
	r.line(d,a,OUTLINE,sha_rot);
	r.line(a,a+z,OUTLINE,sha_rot);
	r.line(b,b+z,OUTLINE,sha_rot);
	r.line(c,c+z,OUTLINE,sha_rot);
	r.line(d,d+z,OUTLINE,sha_rot);
	r.line(a+z,b+z,OUTLINE,sha_rot);
	r.line(b+z,c+z,OUTLINE,sha_rot);
	r.line(c+z,d+z,OUTLINE,sha_rot);
	r.line(d+z,a+z,OUTLINE,sha_rot);
	r.area(a,b,a+z,sha_col,sha_rot);
	r.area(b,c,b+z,sha_col,sha_rot);
	r.area(c+z1,d+z1,c+z,sha_col,sha_rot);
	r.area(d,a,d+z,sha_col,sha_rot);
	r.area(b+z,a+z,c+z,sha_col,sha_rot);
	r.triang(a+z+XUNIT,c+z,d+z,sha_col,sha_rot);
	vector l(c-d);
	vector l1(d+0.106*l);
	vector l2(d+0.284*l);
	vector l3(d+0.39*l);
	vector l4(d+0.61*l);
	r.line(d,l1,OUTLINE,sha_rot);
	r.line(l1,l1+z1,OUTLINE,sha_rot);
	r.line(l1+z1,l2+z1,OUTLINE,sha_rot);
	r.line(l2+z1,l2,OUTLINE,sha_rot);
	r.line(l2,l3,OUTLINE,sha_rot);
	r.line(l3,l3+z1,OUTLINE,sha_rot);
	r.line(l3+z1,l4+z1,OUTLINE,sha_rot);
	r.line(l4+z1,l4,OUTLINE,sha_rot);
	r.line(l4,c,OUTLINE,sha_rot);
	r.area(d,l1,d+z1,sha_col,sha_rot);
	r.area(l2,l3,l2+z1,sha_col,sha_rot);
	r.area(l4,c,l4+z1,sha_col,sha_rot);
	}

void grille::draw(room& r)
	{
	vector x(sha_siz.x(),0,0);
	vector y(0,sha_siz.y(),0);
	vector z(0,0,sha_siz.z());
	vector dx(0.2*x);
	vector dy(0.2*YUNIT);
	vector dz(0.6*ZUNIT);
	vector a = sha_org;
	vector b = a + x;
	vector c = b + y;
	vector d = a + y;
	r.area(a+z,a+dx+z,d+z,sha_col,sha_rot);
	r.area(a+2*dx+z,a+3*dx+z,d+2*dx+z,sha_col,sha_rot);
	r.area(b+z,b-dx+z,c+z,sha_col,sha_rot);
	r.area(a,d,a+z,sha_col,sha_rot);
	r.area(b,c,b+z,sha_col,sha_rot);
	r.area(a,b,a+dz,sha_col,sha_rot);
	r.area(c,d,c+dz,sha_col,sha_rot);
	r.area(a+z,a+z+dx,a+dz,sha_col,sha_rot);
	r.area(d+z,d+z+dx,d+dz,sha_col,sha_rot);
	r.area(b+z,b+z-dx,b+dz,sha_col,sha_rot);
	r.area(c+z,c+z-dx,c+dz,sha_col,sha_rot);
	r.area(a+z+2*dx,a+z+3*dx,a+dz+2*dx,sha_col,sha_rot);
	r.area(d+z+2*dx,d+z+3*dx,d+dz+2*dx,sha_col,sha_rot);
	r.area(a+dx+z,a+dx+dz,d+dx+z,sha_col,sha_rot);
	r.area(b-dx+z,b-dx+dz,c-dx+z,sha_col,sha_rot);
	r.area(a+2*dx+z,a+2*dx+dz,d+2*dx+z,sha_col,sha_rot);
	r.area(b-2*dx+z,b-2*dx+dz,c-2*dx+z,sha_col,sha_rot);
	r.area(a+dx+dz,a+dx+dy+dz,a+2*dx+dz,sha_col,sha_rot);
	r.area(b-dx+dz,b-dx+dy+dz,b-2*dx+dz,sha_col,sha_rot);
	r.area(d+dx+dz,d+dx-dy+dz,d+2*dx+dz,sha_col,sha_rot);
	r.area(c-dx+dz,c-dx-dy+dz,c-2*dx+dz,sha_col,sha_rot);
	r.area(a+dx+dy,a+dx+dy+dz,b-dx+dy,sha_col,sha_rot);
	r.area(d+dx-dy,d+dx-dy+dz,c-dx-dy,sha_col,sha_rot);
	r.area(a+dx+dy,a+dx+dy+dz,d+dx-dy,sha_col,sha_rot);
	r.area(b-dx+dy,b-dx+dy+dz,c-dx-dy,sha_col,sha_rot);
	r.line(a,b,OUTLINE,sha_rot);
	r.line(b,c,OUTLINE,sha_rot);
	r.line(c,d,OUTLINE,sha_rot);
	r.line(d,a,OUTLINE,sha_rot);
	r.line(a,a+z,OUTLINE,sha_rot);
	r.line(b,b+z,OUTLINE,sha_rot);
	r.line(c,c+z,OUTLINE,sha_rot);
	r.line(d,d+z,OUTLINE,sha_rot);
	r.line(a+z,d+z,OUTLINE,sha_rot);
	r.line(b+z,c+z,OUTLINE,sha_rot);
	r.line(a+z,a+z+dx,OUTLINE,sha_rot);
	r.line(b+z,b+z-dx,OUTLINE,sha_rot);
	r.line(a+dx+z,a+dx+dz,OUTLINE,sha_rot);
	r.line(b-dx+z,b-dx+dz,OUTLINE,sha_rot);
	r.line(a+dx+dz,a+2*dx+dz,OUTLINE,sha_rot);
	r.line(b-dx+dz,b-2*dx+dz,OUTLINE,sha_rot);
	r.line(a+2*dx+dz,a+2*dx+z,OUTLINE,sha_rot);
	r.line(b-2*dx+dz,b-2*dx+z,OUTLINE,sha_rot);
	r.line(a+3*dx+z,b-3*dx+z,OUTLINE,sha_rot);
	r.line(d+z,d+z+dx,OUTLINE,sha_rot);
	r.line(c+z,c+z-dx,OUTLINE,sha_rot);
	r.line(d+dx+z,d+dx+dz,OUTLINE,sha_rot);
	r.line(c-dx+z,c-dx+dz,OUTLINE,sha_rot);
	r.line(d+dx+dz,d+2*dx+dz,OUTLINE,sha_rot);
	r.line(c-dx+dz,c-2*dx+dz,OUTLINE,sha_rot);
	r.line(d+2*dx+dz,d+2*dx+z,OUTLINE,sha_rot);
	r.line(c-2*dx+dz,c-2*dx+z,OUTLINE,sha_rot);
	r.line(d+3*dx+z,c-3*dx+z,OUTLINE,sha_rot);
	r.line(a+dx+dy,d+dx-dy,OUTLINE,sha_rot);
	r.line(b-dx+dy,c-dx-dy,OUTLINE,sha_rot);
	r.line(a+dx+dy,b-dx+dy,OUTLINE,sha_rot);
	r.line(d+dx-dy,c-dx-dy,OUTLINE,sha_rot);
	r.line(a+dx+dy,a+dx+dy+dz,OUTLINE,sha_rot);
	r.line(b-dx+dy,b-dx+dy+dz,OUTLINE,sha_rot);
	r.line(c-dx-dy,c-dx-dy+dz,OUTLINE,sha_rot);
	r.line(d+dx-dy,d+dx-dy+dz,OUTLINE,sha_rot);
	r.line(a+dx+dy+dz,b-dx+dy+dz,OUTLINE,sha_rot);
	r.line(c-dx-dy+dz,d+dx-dy+dz,OUTLINE,sha_rot);
	r.line(a+dx+dy+dz,a+dx+dz,OUTLINE,sha_rot);
	r.line(b-dx+dy+dz,b-dx+dz,OUTLINE,sha_rot);
	r.line(c-dx-dy+dz,c-dx+dz,OUTLINE,sha_rot);
	r.line(d+dx-dy+dz,d+dx+dz,OUTLINE,sha_rot);
	r.line(a+2*dx+dz,d+2*dx+dz,OUTLINE,sha_rot);
	r.line(b-2*dx+dz,c-2*dx+dz,OUTLINE,sha_rot);
	r.line(a+2*dx+z,d+2*dx+z,OUTLINE,sha_rot);
	r.line(b-2*dx+z,c-2*dx+z,OUTLINE,sha_rot);
	r.line(a+dx+z,d+dx+z,OUTLINE,sha_rot);
	r.line(b-dx+z,c-dx+z,OUTLINE,sha_rot);
	}

void handle::draw(room& r)
	{
	vector x(sha_siz.x(),0,0);
	vector y(0,sha_siz.y(),0);
	vector delx(0.2*XUNIT);
	const double beta = 147.47959;
	const double rad = 5.0/24.0;
	vector vrad(0,-rad,0);
	vector al (sha_org);
	vector ar (al + x);
	vector ali(al + delx);
	vector ari(ar - delx);
	vector bl (al + y);
	vector br (bl + x);
	vector cl (bl + ZUNIT);
	vector cr (br + ZUNIT);
	vector cli(cl + delx);
	vector cri(cr - delx);
	vector dl (al + ZUNIT);
	vector dr (ar + ZUNIT);
	vector dli(dl + delx);
	vector dri(dr - delx);
	vector el (al + 3.5*ZUNIT);
	vector er (el + x);
	vector eli(el + delx);
	vector eri(er - delx);
	vector fl (el + vector(0,rad*(1-cos(beta*DEGREE)),2.5*rad*sin(beta*DEGREE)));
	vector fr (fl + x);
	vector fli(fl + delx);
	vector fri(fr - delx);
	vector ml (el - vrad);
	vector mr (ml + x);
	vector mli(ml + delx);
	vector mri(mr - delx);
	r.area(bl,al,cl,sha_col,sha_rot);
	r.area(br,ar,cr,sha_col,sha_rot);
	r.area(bl,cl,br,sha_col,sha_rot);
	r.area(ali,dli,ari,sha_col,sha_rot);
	r.area(dli,cli,dri,sha_col,sha_rot);
	r.area(el,eli,al,sha_col,sha_rot);
	r.area(er,eri,ar,sha_col,sha_rot);
	r.area(cl,cli,fl,sha_col,sha_rot);
	r.area(cr,cri,fr,sha_col,sha_rot);
	r.area(el,dl,ml,sha_col,sha_rot);
	r.area(er,dr,mr,sha_col,sha_rot);
	r.area(eli,dli,mli,sha_col,sha_rot);
	r.area(eri,dri,mri,sha_col,sha_rot);
	r.triang(ml,dl-vrad,cl,sha_col,sha_rot);
	r.triang(mr,dr-vrad,cr,sha_col,sha_rot);
	r.triang(mli,dli-vrad,cli,sha_col,sha_rot);
	r.triang(mri,dri-vrad,cri,sha_col,sha_rot);
	r.triang(ml,cl,fl,sha_col,sha_rot);
	r.triang(mr,cr,fr,sha_col,sha_rot);
	r.triang(mli,cli,fli,sha_col,sha_rot);
	r.triang(mri,cri,fri,sha_col,sha_rot);
	r.cyl(mli,mri-mli,rad,sha_col,sha_rot);
	r.cylsector(ml,delx,vrad,-beta,sha_col,sha_rot);
	r.cylsector(mri,delx,vrad,-beta,sha_col,sha_rot);
	r.sector(ml,x,vrad,-beta,sha_col,sha_rot);
	r.sector(mr,x,vrad,-beta,sha_col,sha_rot);
	r.arc(mli,x,vrad,360-beta,OUTLINE,sha_rot);
	r.arc(mri,x,vrad,360-beta,OUTLINE,sha_rot);
	r.arc(ml,x,vrad,-beta,OUTLINE,sha_rot);
	r.arc(mr,x,vrad,-beta,OUTLINE,sha_rot);
	r.line(al,ar,OUTLINE,sha_rot);
	r.line(bl,br,OUTLINE,sha_rot);
	r.line(al,bl,OUTLINE,sha_rot);
	r.line(ar,br,OUTLINE,sha_rot);
	r.line(dli,dri,OUTLINE,sha_rot);
	r.line(cli,cri,OUTLINE,sha_rot);
	r.line(bl,cl,OUTLINE,sha_rot);
	r.line(br,cr,OUTLINE,sha_rot);
	r.line(cl,cr,OUTLINE,sha_rot);
	r.line(cli,dli,OUTLINE,sha_rot);
	r.line(cri,dri,OUTLINE,sha_rot);
	r.line(al,el,OUTLINE,sha_rot);
	r.line(ar,er,OUTLINE,sha_rot);
	r.line(dli,eli,OUTLINE,sha_rot);
	r.line(dri,eri,OUTLINE,sha_rot);
	r.line(cl,fl,OUTLINE,sha_rot);
	r.line(cli,fli,OUTLINE,sha_rot);
	r.line(cr,fr,OUTLINE,sha_rot);
	r.line(cri,fri,OUTLINE,sha_rot);
	}

void towballpart::draw(room& r)
	{
	float r1 = 0.1875;
	float r2 = 0.36875;
	float l = 0.682478;
	vector org(sha_org);
	vector dir(0,1.0,0);
	r.circle(org,dir,r1,OUTLINE,sha_rot);
	r.cyl(org,l*dir,r1,sha_col,sha_rot);
	org += l*dir;
	r.circle(org,dir,r1,OUTLINE,sha_rot);
	org = sha_org + dir;
	r.sphere(org,r2,sha_col,sha_rot);
	}

void towsocketpart::draw(room& r)
	{
	float d1 = 0.35;
	float d2 = 0.57;
	float l1 = 0.075;
	float l2 = 0.27;
	float l3 = 0.85;
	float arc = 20.0;
	vector org(sha_org);
	vector dir(0,1.0,0);
	vector vol(circularXZ(-0.5*arc));
	vector vor(circularXZ(+0.5*arc));
	vector vur(circularXZ(180-0.5*arc));
	vector vul(circularXZ(180+0.5*arc));
	vector vro(circularXZ(90-0.5*arc));
	vector vru(circularXZ(90+0.5*arc));
	vector vlu(circularXZ(-90-0.5*arc));
	vector vlo(circularXZ(-90+0.5*arc));
	vector vol1(vol*d1);
	vector vor1(vor*d1);
	vector vur1(vur*d1);
	vector vul1(vul*d1);
	vector vro1(vro*d1);
	vector vru1(vru*d1);
	vector vlu1(vlu*d1);
	vector vlo1(vlo*d1);
	vector vol2(vol*d2);
	vector vor2(vor*d2);
	vector vur2(vur*d2);
	vector vul2(vul*d2);
	vector vro2(vro*d2);
	vector vru2(vru*d2);
	vector vlu2(vlu*d2);
	vector vlo2(vlo*d2);
	r.circle(org,dir,d2,OUTLINE,sha_rot);
	r.disc(org,dir,d2,sha_col,sha_rot);
	r.cyl(org,dir*l2,d2,sha_col,sha_rot);
	org = sha_org + dir*l1;
	r.circle(org,dir,d1,OUTLINE,sha_rot);
	r.disc(org,dir,d1,sha_col,sha_rot);
	r.cyl(org,dir*(l2-l1),d1,sha_col,sha_rot);
	org = sha_org + dir*l2;
	r.arc(org,dir,vol2,arc,OUTLINE,sha_rot);
	r.arc(org,dir,vlu2,arc,OUTLINE,sha_rot);
	r.arc(org,dir,vur2,arc,OUTLINE,sha_rot);
	r.arc(org,dir,vro2,arc,OUTLINE,sha_rot);
	r.arc(org,dir,vol1,arc,OUTLINE,sha_rot);
	r.arc(org,dir,vlu1,arc,OUTLINE,sha_rot);
	r.arc(org,dir,vur1,arc,OUTLINE,sha_rot);
	r.arc(org,dir,vro1,arc,OUTLINE,sha_rot);
	r.ringsector(org,dir,vol1,vol2,arc,sha_col,sha_rot);
	r.ringsector(org,dir,vlu1,vlu2,arc,sha_col,sha_rot);
	r.ringsector(org,dir,vur1,vur2,arc,sha_col,sha_rot);
	r.ringsector(org,dir,vro1,vro2,arc,sha_col,sha_rot);
	vector shift(dir*(l3-l2));
	r.cylsector(org,shift,vor2,90-arc,sha_col,sha_rot);
	r.cylsector(org,shift,vlo2,90-arc,sha_col,sha_rot);
	r.cylsector(org,shift,vul2,90-arc,sha_col,sha_rot);
	r.cylsector(org,shift,vru2,90-arc,sha_col,sha_rot);
	r.cylsector(org,shift,vor1,90-arc,sha_col,sha_rot);
	r.cylsector(org,shift,vlo1,90-arc,sha_col,sha_rot);
	r.cylsector(org,shift,vul1,90-arc,sha_col,sha_rot);
	r.cylsector(org,shift,vru1,90-arc,sha_col,sha_rot);
	r.line(org+vor1,org+vor2,OUTLINE,sha_rot);
	r.line(org+vol1,org+vol2,OUTLINE,sha_rot);
	r.line(org+vur1,org+vur2,OUTLINE,sha_rot);
	r.line(org+vul1,org+vul2,OUTLINE,sha_rot);
	r.line(org+vro1,org+vro2,OUTLINE,sha_rot);
	r.line(org+vru1,org+vru2,OUTLINE,sha_rot);
	r.line(org+vlo1,org+vlo2,OUTLINE,sha_rot);
	r.line(org+vlu1,org+vlu2,OUTLINE,sha_rot);
	r.line(org+vor2,org+vor2+shift,OUTLINE,sha_rot);
	r.line(org+vol2,org+vol2+shift,OUTLINE,sha_rot);
	r.line(org+vur2,org+vur2+shift,OUTLINE,sha_rot);
	r.line(org+vul2,org+vul2+shift,OUTLINE,sha_rot);
	r.line(org+vlo2,org+vlo2+shift,OUTLINE,sha_rot);
	r.line(org+vlu2,org+vlu2+shift,OUTLINE,sha_rot);
	r.line(org+vro2,org+vro2+shift,OUTLINE,sha_rot);
	r.line(org+vru2,org+vru2+shift,OUTLINE,sha_rot);
	r.line(org+vor1,org+vor1+shift,OUTLINE,sha_rot);
	r.line(org+vol1,org+vol1+shift,OUTLINE,sha_rot);
	r.line(org+vur1,org+vur1+shift,OUTLINE,sha_rot);
	r.line(org+vul1,org+vul1+shift,OUTLINE,sha_rot);
	r.line(org+vlo1,org+vlo1+shift,OUTLINE,sha_rot);
	r.line(org+vlu1,org+vlu1+shift,OUTLINE,sha_rot);
	r.line(org+vro1,org+vro1+shift,OUTLINE,sha_rot);
	r.line(org+vru1,org+vru1+shift,OUTLINE,sha_rot);
	r.area(org+vor1,org+vor1+shift,org+vor2,sha_col,sha_rot);
	r.area(org+vol1,org+vol1+shift,org+vol2,sha_col,sha_rot);
	r.area(org+vur1,org+vur1+shift,org+vur2,sha_col,sha_rot);
	r.area(org+vul1,org+vul1+shift,org+vul2,sha_col,sha_rot);
	r.area(org+vlo1,org+vlo1+shift,org+vlo2,sha_col,sha_rot);
	r.area(org+vlu1,org+vlu1+shift,org+vlu2,sha_col,sha_rot);
	r.area(org+vro1,org+vro1+shift,org+vro2,sha_col,sha_rot);
	r.area(org+vru1,org+vru1+shift,org+vru2,sha_col,sha_rot);
	org = sha_org + dir*l3;
	r.arc(org,dir,vor2,90-arc,OUTLINE,sha_rot);
	r.arc(org,dir,vlo2,90-arc,OUTLINE,sha_rot);
	r.arc(org,dir,vul2,90-arc,OUTLINE,sha_rot);
	r.arc(org,dir,vru2,90-arc,OUTLINE,sha_rot);
	r.arc(org,dir,vor1,90-arc,OUTLINE,sha_rot);
	r.arc(org,dir,vlo1,90-arc,OUTLINE,sha_rot);
	r.arc(org,dir,vul1,90-arc,OUTLINE,sha_rot);
	r.arc(org,dir,vru1,90-arc,OUTLINE,sha_rot);
	r.ringsector(org,dir,vor1,vor2,90-arc,sha_col,sha_rot);
	r.ringsector(org,dir,vlo1,vlo2,90-arc,sha_col,sha_rot);
	r.ringsector(org,dir,vul1,vul2,90-arc,sha_col,sha_rot);
	r.ringsector(org,dir,vru1,vru2,90-arc,sha_col,sha_rot);
	r.line(org+vor1,org+vor2,OUTLINE,sha_rot);
	r.line(org+vol1,org+vol2,OUTLINE,sha_rot);
	r.line(org+vur1,org+vur2,OUTLINE,sha_rot);
	r.line(org+vul1,org+vul2,OUTLINE,sha_rot);
	r.line(org+vlo1,org+vlo2,OUTLINE,sha_rot);
	r.line(org+vlu1,org+vlu2,OUTLINE,sha_rot);
	r.line(org+vro1,org+vro2,OUTLINE,sha_rot);
	r.line(org+vru1,org+vru2,OUTLINE,sha_rot);
	}

void fence1part::draw(room& r)
	{
	color DARK(sha_col.dark());
	vector top1(sha_org+vector(0,0,0.6));
	vector top2(sha_org+sha_siz);
	r.ring(top1,sha_siz,0.18,0.3,sha_col,sha_rot);
	r.circle(sha_org,sha_siz,0.3,OUTLINE,sha_rot);
	r.circle(top1,sha_siz,0.3,OUTLINE,sha_rot);
	r.circle(top1,sha_siz,0.18,OUTLINE,sha_rot);
	r.circle(top2,sha_siz,0.18,OUTLINE,sha_rot);
	r.cyl(sha_org,vector(0,0,0.6),0.3,DARK,sha_rot);
	r.cyl(top1,top2-top1,0.18,sha_col,sha_rot);
	}

void fence2part::draw(room& r)
	{
	vector h(0,0,sha_siz.z()-sha_siz.y()*1.25);
	vector abot(sha_org);
	vector atop(abot+h);
	vector bbot(abot+vector(sha_siz.x(),0,0));
	vector btop(bbot+h);
	vector cbot(bbot+vector(0,sha_siz.y(),0));
	vector ctop(cbot+h);
	vector dbot(abot+vector(0,sha_siz.y(),0));
	vector dtop(dbot+h);
	vector mleft((atop+dtop)*0.5);
	vector mright((btop+ctop)*0.5);
	r.line(abot,atop,OUTLINE,sha_rot);
	r.line(bbot,btop,OUTLINE,sha_rot);
	r.line(cbot,ctop,OUTLINE,sha_rot);
	r.line(dbot,dtop,OUTLINE,sha_rot);
	r.line(abot,bbot,OUTLINE,sha_rot);
	r.line(bbot,cbot,OUTLINE,sha_rot);
	r.line(cbot,dbot,OUTLINE,sha_rot);
	r.line(dbot,abot,OUTLINE,sha_rot);
	r.arc(mleft,XUNIT,dtop-mleft,180,OUTLINE,sha_rot);
	r.arc(mright,XUNIT,ctop-mright,180,OUTLINE,sha_rot);
	r.area(abot,atop,bbot,sha_col,sha_rot);
	r.area(abot,atop,dbot,sha_col,sha_rot);
	r.area(cbot,ctop,bbot,sha_col,sha_rot);
	r.area(cbot,ctop,dbot,sha_col,sha_rot);
	r.cylsector(mleft,XUNIT*sha_siz.x(),ctop-mright,180,sha_col,sha_rot);
	r.sector(mleft,XUNIT,ctop-mright,180,sha_col,sha_rot);
	r.sector(mright,XUNIT,ctop-mright,180,sha_col,sha_rot);
	}

void testbody::draw(room& r)
	{
/*	double arc = 120.0;
	vector org,dir,rad;

	org = sha_org+vector(-4,4,0);
	dir = vector(1,0,0);
	rad = vector(0,0,3);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);
	
	org = sha_org+vector(-1.75,4,0);
	dir = vector(1,0,0);
	rad = vector(0,0,-3);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);

	org = sha_org+vector(+1.75,4,0);
	dir = vector(-1,0,0);
	rad = vector(0,0,3);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);
	
	org = sha_org+vector(+4,4,0);
	dir = vector(-1,0,0);
	rad = vector(0,0,-3);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);


	org = sha_org+vector(-4,0,0);
	dir = vector(0,1,0);
	rad = vector(1,0,0);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);
	
	org = sha_org+vector(-1.25,0,0);
	dir = vector(0,1,0);
	rad = vector(-1,0,0);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);

	org = sha_org+vector(+1.25,0,0);
	dir = vector(0,-1,0);
	rad = vector(1,0,0);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);
	
	org = sha_org+vector(+4,0,0);
	dir = vector(0,-1,0);
	rad = vector(-1,0,0);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);


	org = sha_org+vector(-4,-4,0);
	dir = vector(0,0,3);
	rad = vector(0,1,0);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);
	
	org = sha_org+vector(-1.5,-4,0);
	dir = vector(0,0,3);
	rad = vector(0,-1,0);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);

	org = sha_org+vector(+1.5,-4,0);
	dir = vector(0,0,-3);
	rad = vector(0,1,0);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);
	
	org = sha_org+vector(+4,-4,0);
	dir = vector(0,0,-3);
	rad = vector(0,-1,0);
	r.line(org,org+dir,OUTLINE,sha_rot);
	r.line(org,org+rad,OUTLINE,sha_rot);
	r.line(org+dir,org+dir+rad,OUTLINE,sha_rot);
	r.cylsector(org,dir,rad,arc,sha_col,sha_rot);
	r.cylsector(org,dir,rad,-arc,sha_col,sha_rot);
*/
	}
	
//
